@@ -66,14 +66,14 @@ static ObjectChange* annotation_move(Annotation *annotation, Point *to);
 static void annotation_select(Annotation *annotation, Point *clicked_point,
 			      DiaRenderer *interactive_renderer);
 static void annotation_draw(Annotation *annotation, DiaRenderer *renderer);
-static Object *annotation_create(Point *startpoint,
+static DiaObject *annotation_create(Point *startpoint,
 				 void *user_data,
 				 Handle **handle1,
 				 Handle **handle2);
 static real annotation_distance_from(Annotation *annotation, Point *point);
 static void annotation_update_data(Annotation *annotation);
 static void annotation_destroy(Annotation *annotation);
-static Object *annotation_load(ObjectNode obj_node, int version,
+static DiaObject *annotation_load(ObjectNode obj_node, int version,
 			       const char *filename);
 static PropDescription *annotation_describe_props(Annotation *annotation);
 static void annotation_get_props(Annotation *annotation, 
@@ -90,7 +90,7 @@ static ObjectTypeOps annotation_type_ops =
   (ApplyDefaultsFunc) NULL
 };
 
-ObjectType sadtannotation_type =
+DiaObjectType sadtannotation_type =
 {
   "SADT - annotation",        /* name */
   0,                         /* version */
@@ -107,11 +107,13 @@ static ObjectOps annotation_ops = {
   (MoveFunc)            annotation_move,
   (MoveHandleFunc)      annotation_move_handle,
   (GetPropertiesFunc)   object_create_props_dialog,
-  (ApplyPropertiesFunc) object_apply_props_from_dialog,
+  (ApplyPropertiesDialogFunc) object_apply_props_from_dialog,
   (ObjectMenuFunc)      NULL,
   (DescribePropsFunc)   annotation_describe_props,
   (GetPropsFunc)        annotation_get_props,
-  (SetPropsFunc)        annotation_set_props
+  (SetPropsFunc)        annotation_set_props,
+  (TextEditFunc) 0,
+  (ApplyPropertiesListFunc) object_apply_props,
 };
 
 #undef TEMPORARY_EVENT_TEST
@@ -165,7 +167,7 @@ static PropOffset annotation_offsets[] = {
   {"text",PROP_TYPE_TEXT,offsetof(Annotation,text)},
   {"text_alignment",PROP_TYPE_ENUM,offsetof(Annotation,attrs.alignment)},
   {"text_font",PROP_TYPE_FONT,offsetof(Annotation,attrs.font)},
-  {"text_height",PROP_TYPE_REAL,offsetof(Annotation,attrs.height)},
+  {PROP_STDNAME_TEXT_HEIGHT,PROP_STDTYPE_TEXT_HEIGHT,offsetof(Annotation,attrs.height)},
   {"text_colour",PROP_TYPE_COLOUR,offsetof(Annotation,attrs.color)},
   {"pos", PROP_TYPE_POINT, offsetof(Annotation,text_handle.pos)},
   { NULL,0,0 }
@@ -322,7 +324,7 @@ annotation_draw(Annotation *annotation, DiaRenderer *renderer)
   text_draw(annotation->text,renderer);
 }
 
-static Object *
+static DiaObject *
 annotation_create(Point *startpoint,
 		  void *user_data,
 		  Handle **handle1,
@@ -331,7 +333,7 @@ annotation_create(Point *startpoint,
   Annotation *annotation;
   Connection *conn;
   LineBBExtras *extra;
-  Object *obj; 
+  DiaObject *obj; 
   Point offs;
   Point defaultlen = { 1.0, 1.0 };
   DiaFont* font;
@@ -396,9 +398,13 @@ static void
 annotation_update_data(Annotation *annotation)
 {
   Connection *conn = &annotation->connection;
-  Object *obj = &conn->object;
+  DiaObject *obj = &conn->object;
   Rectangle textrect;
   
+  if (connpoint_is_autogap(conn->endpoint_handles[0].connected_to) ||
+      connpoint_is_autogap(conn->endpoint_handles[1].connected_to)) {
+    connection_adjust_for_autogap(conn);
+  }
   obj->position = conn->endpoints[0];
 
   annotation->text_handle.pos = annotation->text->position;
@@ -410,7 +416,7 @@ annotation_update_data(Annotation *annotation)
   rectangle_union(&obj->bounding_box, &textrect);
 }
 
-static Object *
+static DiaObject *
 annotation_load(ObjectNode obj_node, int version, const char *filename)
 {
   return object_load_using_properties(&sadtannotation_type,
@@ -425,3 +431,4 @@ annotation_load(ObjectNode obj_node, int version, const char *filename)
 
 
 
+
