@@ -69,13 +69,14 @@ typedef struct _Sadtarrow {
   gboolean autogray;
 } Sadtarrow;
 
-static void sadtarrow_move_handle(Sadtarrow *sadtarrow, Handle *handle,
-				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void sadtarrow_move(Sadtarrow *sadtarrow, Point *to);
+static ObjectChange* sadtarrow_move_handle(Sadtarrow *sadtarrow, Handle *handle,
+					   Point *to, ConnectionPoint *cp,
+					   HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* sadtarrow_move(Sadtarrow *sadtarrow, Point *to);
 static void sadtarrow_select(Sadtarrow *sadtarrow, Point *clicked_point,
 			      DiaRenderer *interactive_renderer);
 static void sadtarrow_draw(Sadtarrow *sadtarrow, DiaRenderer *renderer);
-static Object *sadtarrow_create(Point *startpoint,
+static DiaObject *sadtarrow_create(Point *startpoint,
 				 void *user_data,
 				 Handle **handle1,
 				 Handle **handle2);
@@ -85,7 +86,7 @@ static void sadtarrow_destroy(Sadtarrow *sadtarrow);
 static DiaMenu *sadtarrow_get_object_menu(Sadtarrow *sadtarrow,
 					   Point *clickedpoint);
 
-static Object *sadtarrow_load(ObjectNode obj_node, int version,
+static DiaObject *sadtarrow_load(ObjectNode obj_node, int version,
 			       const char *filename);
 static PropDescription *sadtarrow_describe_props(Sadtarrow *sadtarrow);
 static void sadtarrow_get_props(Sadtarrow *sadtarrow, 
@@ -103,7 +104,7 @@ static ObjectTypeOps sadtarrow_type_ops =
   (ApplyDefaultsFunc) NULL
 };
 
-ObjectType sadtarrow_type =
+DiaObjectType sadtarrow_type =
 {
   "SADT - arrow",   /* name */
   0,                         /* version */
@@ -122,11 +123,13 @@ static ObjectOps sadtarrow_ops = {
   (MoveFunc)            sadtarrow_move,
   (MoveHandleFunc)      sadtarrow_move_handle,
   (GetPropertiesFunc)   object_create_props_dialog,
-  (ApplyPropertiesFunc) object_apply_props_from_dialog,
+  (ApplyPropertiesDialogFunc) object_apply_props_from_dialog,
   (ObjectMenuFunc)      sadtarrow_get_object_menu,
   (DescribePropsFunc)   sadtarrow_describe_props,
   (GetPropsFunc)        sadtarrow_get_props,
-  (SetPropsFunc)        sadtarrow_set_props
+  (SetPropsFunc)        sadtarrow_set_props,
+  (TextEditFunc) 0,
+  (ApplyPropertiesListFunc) object_apply_props,
 };
 
 PropEnumData flow_style[] = {
@@ -194,28 +197,35 @@ sadtarrow_select(Sadtarrow *sadtarrow, Point *clicked_point,
   neworthconn_update_data(&sadtarrow->orth);
 }
 
-static void
+static ObjectChange*
 sadtarrow_move_handle(Sadtarrow *sadtarrow, Handle *handle,
-		       Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		      Point *to, ConnectionPoint *cp,
+		      HandleMoveReason reason, ModifierKeys modifiers)
 {
+  ObjectChange *change;
   assert(sadtarrow!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
-  neworthconn_move_handle(&sadtarrow->orth, handle, to, reason);
+  change = neworthconn_move_handle(&sadtarrow->orth, handle, to, cp, 
+				   reason, modifiers);
   sadtarrow_update_data(sadtarrow);
+
+  return change;
 }
 
 
-static void
+static ObjectChange*
 sadtarrow_move(Sadtarrow *sadtarrow, Point *to)
 {
-  neworthconn_move(&sadtarrow->orth, to);
+  ObjectChange *change;
+
+  change = neworthconn_move(&sadtarrow->orth, to);
   sadtarrow_update_data(sadtarrow);
+
+  return change;
 }
 
-static void draw_arrowhead(DiaRenderer *renderer,
-			   Point *end, Point *vect, Color *col);
 static void draw_dot(DiaRenderer *renderer,
 		     Point *end, Point *vect, Color *col);
 static void draw_tunnel(DiaRenderer *renderer,
@@ -231,10 +241,8 @@ sadtarrow_draw(Sadtarrow *sadtarrow, DiaRenderer *renderer)
   NewOrthConn *orth = &sadtarrow->orth;
   Point *points;
   int n;
-  int i;
-  Point *p;
-  real zzr;
   Color col;
+  Arrow arrow;
 
   points = &orth->points[0];
   n = orth->numpoints;
@@ -252,138 +260,34 @@ sadtarrow_draw(Sadtarrow *sadtarrow, DiaRenderer *renderer)
     col.blue = GBASE + (GMULT*col.blue);
   }
 
-  /* Now we draw what's in essence a rounded-corner zig-zag line */
-  p = points;
-  zzr = ARROW_CORNER_RADIUS;
+  arrow.type = ARROW_HEAD_TYPE;
+  arrow.length = ARROW_HEAD_LENGTH;
+  arrow.width = ARROW_HEAD_WIDTH;
 
-  /* This code would welcome some optimisations. In the meantime, it sorta 
-     works. */
-  for (i=0; i<(n-2); i++,p++) 
-    {
-      real len1,len2;
-      real rr,cosa,sina,cosb,sinb,/*cosg,*/sing,alpha,beta,/*gamma,*/ca,cb;
-      Point v1,v2,vca,vcb;
-      Point X,Y,m1,m2,M,C,A,B;
-      
-      rr = zzr;
-      
-      len1 = distance_point_point(p,p+1);
-      if ((len1/2) < rr) rr = len1/2;
-      len2 = distance_point_point(p+1,p+2);
-      if ((len2/2) < rr) rr = len2/2;
-
-      M.x = (*(p+1)).x; X.x = p->x; Y.x = (*(p+2)).x;
-      M.y = (*(p+1)).y; X.y = p->y; Y.y = (*(p+2)).y;
-      m1.x = (i==0)?(p->x):((p->x + M.x) / 2);
-      m1.y = (i==0)?(p->y):((p->y + M.y) / 2);
-      m2.x = (i==(n-3))?(Y.x):((M.x + Y.x) / 2);
-      m2.y = (i==(n-3))?(Y.y):((M.y + Y.y) / 2);
-      
-      if (rr < .01) {
-	/* too small corner : we do it a bit faster */
-	renderer_ops->draw_line(renderer,&m1,&M,&col);
-	renderer_ops->draw_line(renderer,&M,&m2,&col);
-       } else {
-         /* full rendering. We know len1 and len2 are nonzero. */
-         v1.x = (M.x - X.x) / len1;
-         v1.y = (M.y - X.y) / len1;
-         v2.x = (Y.x - M.x) / len2;
-         v2.y = (Y.y - M.y) / len2;
-       
-         A.x = M.x - (v1.x * rr);
-         A.y = M.y - (v1.y * rr);
-         renderer_ops->draw_line(renderer,&m1,&A,&col);
-         B.x = M.x + (v2.x * rr);
-         B.y = M.y + (v2.y * rr);
-         renderer_ops->draw_line(renderer,&B,&m2,&col);
-         C.x = A.x + (v2.x * rr); /* (or B.x - v1.x*rr) */
-         C.y = A.y + (v2.y * rr);
-         
-         vca.x = A.x - C.x;
-         vca.y = -(A.y - C.y); /* @#$! coordinate system */
-         vcb.x = B.x - C.x;
-         vcb.y = -(B.y - C.y);
-         
-         ca = distance_point_point(&C,&A);
-	 cb = distance_point_point(&C,&B);
-	 
-	 if ((ca > 1E-7) && (cb > 1E-7)) {
-	   cosa = vca.x / ca; sina = vca.y / ca;
-	   cosb = vcb.x / cb; sinb = vcb.y / cb;
-	   /* Seems that sometimes vca.x > ca etc (though it should be
-	    * impossible), so we make border cases here */
-	   if (cosa > 1.0) cosa = 1.0;
-	   if (cosa < -1.0) cosa = -1.0;
-	   if (cosb > 1.0) cosa = 1.0;
-	   if (cosb < -1.0) cosb = -1.0;
-	   /*cosg = ((M.x-X.x)*(Y.x-M.x) + (M.y-X.y)*(Y.y-M.y))/(len1*len2);*/
-	   sing = (-(M.x-X.x)*(Y.y-M.y) + (Y.x-M.x)*(M.y-X.y)) / (len1*len2);
-
-	   alpha = acos(cosa) * 180.0 / M_PI;
-	   if (sina < 0.0) alpha = -alpha;
-	   beta = acos(cosb) * 180.0 / M_PI ; 
-	   if (sinb < 0.0) beta = -beta;
-	   /* we'll keep gamma in radians, since we're only interested in its
-	      sign */
-	   /* after all, we don't even need to compute cos(gamma), and
-	      we can use sin(gamma) to extract the sign of gamma itself. */
-         
-	   if (alpha < 0.0) alpha += 360.0;
-	   if (beta < 0.0) beta += 360.0;
-	   if (sing /* gamma*/ < 0) {
-	     /* if this happens, we swap alpha and beta, in order to draw
-		the other portion of circle */
-	     real tau;
-	     tau = beta;
-	     beta=alpha;
-	     alpha = tau;
-	   }
-
-	   renderer_ops->draw_arc(renderer,&C,rr*2,rr*2,alpha,beta,
-				   &col);
-	 }
-       }
-    }
-  
-  /* depending on the exact arrow type, we'll draw different gizmos 
-     (arrow heads, dots, tunnel) at different places. */
-
-  /* XXX : chop off a little (.1) the starting and ending segments, so that
-     their ends don't show up in front of the arrow heads ? Ugh.
-  */
+  renderer_ops->draw_rounded_polyline_with_arrows
+    (renderer, points, n, ARROW_LINE_WIDTH, &col,
+     sadtarrow->style == SADT_ARROW_DOTTED?&arrow:NULL,
+     sadtarrow->style != SADT_ARROW_DISABLED?&arrow:NULL,
+     ARROW_CORNER_RADIUS);
 
+  /* Draw the extra stuff. */
   switch (sadtarrow->style) {
-  case SADT_ARROW_NORMAL:
-    draw_arrowhead(renderer,&points[n-1], &points[n-2],&col);
-    break;
   case SADT_ARROW_IMPORTED:
-    draw_arrowhead(renderer,&points[n-1], &points[n-2],&col);
     draw_tunnel(renderer,&points[0],&points[1],&col);
     break;
   case SADT_ARROW_IMPLIED:
-    draw_arrowhead(renderer,&points[n-1], &points[n-2],&col);
     draw_tunnel(renderer,&points[n-1],&points[n-2],&col);
     break;
   case SADT_ARROW_DOTTED:
-    draw_arrowhead(renderer,&points[n-1], &points[n-2],&col);
-    draw_arrowhead(renderer,&points[0], &points[1],&col);
     draw_dot(renderer,&points[n-1], &points[n-2],&col);
     draw_dot(renderer,&points[0], &points[1],&col);
     break;
+  case SADT_ARROW_NORMAL:
   case SADT_ARROW_DISABLED:
-    /* No arrow heads */
     break;
   }
 }
-static void draw_arrowhead(DiaRenderer *renderer,
-		       Point *end, Point *vect, Color *col)
-{
-    arrow_draw(renderer, ARROW_HEAD_TYPE,
-	       end, vect,
-	       ARROW_HEAD_LENGTH, ARROW_HEAD_WIDTH, 
-	       ARROW_LINE_WIDTH,
-	       col, &color_white);
-}
+
 static void draw_dot(DiaRenderer *renderer,
 		     Point *end, Point *vect, Color *col)
 {
@@ -459,8 +363,7 @@ static void draw_tunnel(DiaRenderer *renderer,
 }
 
 
-
-static Object *
+static DiaObject *
 sadtarrow_create(Point *startpoint,
 		  void *user_data,
 		  Handle **handle1,
@@ -468,7 +371,7 @@ sadtarrow_create(Point *startpoint,
 {
   Sadtarrow *sadtarrow;
   NewOrthConn *orth;
-  Object *obj;
+  DiaObject *obj;
 
   sadtarrow = g_malloc0(sizeof(Sadtarrow));
   orth = &sadtarrow->orth;
@@ -535,7 +438,7 @@ sadtarrow_update_data(Sadtarrow *sadtarrow)
 }
 
 static ObjectChange *
-sadtarrow_add_segment_callback(Object *obj, Point *clicked, gpointer data)
+sadtarrow_add_segment_callback(DiaObject *obj, Point *clicked, gpointer data)
 {
   ObjectChange *change;
   change = neworthconn_add_segment((NewOrthConn *)obj, clicked);
@@ -544,7 +447,7 @@ sadtarrow_add_segment_callback(Object *obj, Point *clicked, gpointer data)
 }
 
 static ObjectChange *
-sadtarrow_delete_segment_callback(Object *obj, Point *clicked, gpointer data)
+sadtarrow_delete_segment_callback(DiaObject *obj, Point *clicked, gpointer data)
 {
   ObjectChange *change;
   change = neworthconn_delete_segment((NewOrthConn *)obj, clicked);
@@ -555,6 +458,7 @@ sadtarrow_delete_segment_callback(Object *obj, Point *clicked, gpointer data)
 static DiaMenuItem object_menu_items[] = {
   { N_("Add segment"), sadtarrow_add_segment_callback, NULL, 1 },
   { N_("Delete segment"), sadtarrow_delete_segment_callback, NULL, 1 },
+  /*  ORTHCONN_COMMON_MENUS,*/
 };
 
 static DiaMenu object_menu = {
@@ -573,10 +477,11 @@ sadtarrow_get_object_menu(Sadtarrow *sadtarrow, Point *clickedpoint)
   /* Set entries sensitive/selected etc here */
   object_menu_items[0].active = neworthconn_can_add_segment(orth, clickedpoint);
   object_menu_items[1].active = neworthconn_can_delete_segment(orth, clickedpoint);
+  /*  orthconn_update_object_menu(orth, clickedpoint, &object_menu_items[2]);*/
   return &object_menu;
 }
 
-static Object *
+static DiaObject *
 sadtarrow_load(ObjectNode obj_node, int version, const char *filename)
 {
   return object_load_using_properties(&sadtarrow_type,
@@ -605,5 +510,6 @@ sadtarrow_load(ObjectNode obj_node, int version, const char *filename)
 
 
 
+
 
 
