@@ -122,11 +122,13 @@ static ObjectOps sadtbox_ops = {
   (MoveFunc)            sadtbox_move,
   (MoveHandleFunc)      sadtbox_move_handle,
   (GetPropertiesFunc)   object_create_props_dialog,
-  (ApplyPropertiesFunc) object_apply_props_from_dialog,
+  (ApplyPropertiesDialogFunc) object_apply_props_from_dialog,
   (ObjectMenuFunc)      sadtbox_get_object_menu,
   (DescribePropsFunc)   sadtbox_describe_props,
   (GetPropsFunc)        sadtbox_get_props,
-  (SetPropsFunc)        sadtbox_set_props
+  (SetPropsFunc)        sadtbox_set_props,
+  (TextEditFunc) 0,
+  (ApplyPropertiesListFunc) object_apply_props,
 };
 
 static PropNumData text_padding_data = { 0.0, 10.0, 0.1 };
@@ -165,7 +167,7 @@ static PropOffset box_offsets[] = {
   { "text", PROP_TYPE_TEXT, offsetof(Box,text)},
   { "text_alignment",PROP_TYPE_ENUM,offsetof(Box,attrs.alignment)},
   { "text_font",PROP_TYPE_FONT,offsetof(Box,attrs.font)},
-  { "text_height",PROP_TYPE_REAL,offsetof(Box,attrs.height)},
+  { PROP_STDNAME_TEXT_HEIGHT,PROP_STDTYPE_TEXT_HEIGHT,offsetof(Box,attrs.height)},
   { "text_colour",PROP_TYPE_COLOUR,offsetof(Box,attrs.color)},
   { "id", PROP_TYPE_STRING, offsetof(Box,id)},
   { "cpl_north",PROP_TYPE_CONNPOINT_LINE, offsetof(Box,north)},
@@ -366,7 +368,9 @@ sadtbox_update_data(Box *box, AnchorShape horiz, AnchorShape vert)
 
   /* Update connections: */
   nw = elem->corner;
-  se = bottom_right;
+  se.x = elem->corner.x + elem->width;
+  se.y = elem->corner.y + elem->height;
+
   ne.x = se.x;
   ne.y = nw.y;
   sw.y = se.y;
@@ -504,6 +508,7 @@ sadtbox_create(Point *startpoint,
                        &color_black,
                        ALIGN_CENTER);
   dia_font_unref(font);
+  text_get_attributes(box->text,&box->attrs);
   
   box->id = g_strdup("A0"); /* should be made better. 
                                Automatic counting ? */
@@ -551,3 +556,4 @@ sadtbox_load(ObjectNode obj_node, int version, const char *filename)
 
 
 
+
